@@ -142,7 +142,7 @@ proc real_file_report_socket {rep} {
         return -2
     }
 
-    puts $s "EDIT $Tkeditpr(prid)"
+    gnatsd_send $s "EDIT $Tkeditpr(prid)"
     set rep [get_socket_reply $s]
     if {![string match 2* [lindex $rep 0]]} {
         set errs "GNATSD error sending EDIT command for PRID $Tkeditpr(prid):\n[join $rep \n]"
@@ -150,7 +150,7 @@ proc real_file_report_socket {rep} {
     }
 
     write_pr $s
-#write_pr stdout
+    #write_pr stdout
     puts     $s "."
 
     set rep [get_socket_reply $s]
@@ -165,7 +165,8 @@ proc real_file_report_socket {rep} {
 proc real_file_report {} {
     global TkGnats Tkeditpr flds frm errorCode whyText
     flush_singletext $Tkeditpr(singletextflds)
-    flush_multitext
+    flush_singletext $Tkeditpr(shorttextflds) .eboxs.shortones
+    flush_multitext 
 
     #
     # do some local field checking..
@@ -174,7 +175,7 @@ proc real_file_report {} {
     headingMsg "Checking fields..." 0
 
     # Check mandatory fields for non-blank input
-    foreach f [concat $Tkeditpr(radioflds) $Tkeditpr(listboxflds) $Tkeditpr(singletextflds) $Tkeditpr(multitextflds)] {
+    foreach f [concat $Tkeditpr(radioflds) $Tkeditpr(listboxflds) $Tkeditpr(singletextflds) $Tkeditpr(multitextflds) $Tkeditpr(shorttextflds)] {
         if {[check_suppressed_field $f]} {
             continue
         }
@@ -238,13 +239,13 @@ proc real_file_report {} {
         }
     }
 
-    foreach f $Tkeditpr(singletextflds) {
+    foreach f [concat $Tkeditpr(singletextflds) $Tkeditpr(shorttextflds)] {
 	# Check that singletext text fields do not have a | char. (not allowed in gnats index)
         if {[string first "|" $frm($f)] >= 0} {
             Msg " '|' is an illegal character for the '[get_field_alias $f]' field!"
             return -1
         }
-	#dputs "Comparing frm($f) and flds($f): $frm($f) vs $flds($f)"
+	dputs "Comparing frm($f) and flds($f): $frm($f) vs $flds($f)"
         if {$frm($f) != $flds($f)} {
             foreach require {Audit Email Reason} {
                 if {[check_audit_trail_opts $require $f]} {
@@ -334,7 +335,9 @@ proc real_file_report {} {
         # Send to the responsible person, original sender and the X-GNATS-Notify list
         merge_into_list mail_list $responsible_addr    $TkGnats(EmailAddr)
         merge_into_list mail_list $flds(Reply-To)      $TkGnats(EmailAddr)
-        merge_into_list mail_list $frm(X-GNATS-Notify) $TkGnats(EmailAddr)
+	if {[info exists $frm(X-GNATS-Notify)]} {
+	    merge_into_list mail_list $frm(X-GNATS-Notify) $TkGnats(EmailAddr)
+	}
         if {$frm(>Responsible) != $flds(>Responsible)} {
             # Mail to the old responsible person too
             merge_into_list mail_list $old_responsible_addr $TkGnats(EmailAddr)
@@ -417,30 +420,36 @@ proc write_pr {fout} {
     #
     # for each parsed field from the PR form...
     #
-#puts "tags:$Tkeditpr(parsed_flds)"
+    #puts "tags:$Tkeditpr(parsed_flds)"
+
     foreach tag $Tkeditpr(parsed_flds) {
         case $tag {_prefix_} {
             #
             #   The mail header, stored under the _prefix_ tag, is written out
             #   unadulterated, except for the edited X-GNATS-Notify field.
             #
-#puts "prefix before:$flds($tag)"
+	    #puts "prefix before:$flds($tag)"
             set lines [split $flds($tag) \n]
             set idx   [lsearch -regexp $lines "^X-GNATS-Notify:"]
-            set repto "X-GNATS-Notify: [lindex [split $frm(X-GNATS-Notify) ,] 0]"
-            foreach addr [lrange [split $frm(X-GNATS-Notify) ,] 1 end] {
-                append repto ", [string trim $addr]"
-            }
-            set lines [lreplace $lines $idx [expr $idx + $flds(_prefix_len_X-GNATS-Notify) - 1] $repto\n]
+	    
+	    if {[info exists frm(X-GNATS-Notify)]} {
+                set repto "X-GNATS-Notify: [lindex [split $frm(X-GNATS-Notify) ,] 0]"
+		foreach addr [lrange [split $frm(X-GNATS-Notify) ,] 1 end] {
+		    append repto ", [string trim $addr]"
+		}
+		set lines [lreplace $lines $idx [expr $idx + $flds(_prefix_len_X-GNATS-Notify) - 1] $repto\n]
+	    }
             #set frm($tag) "[string trim [join $lines \n]]\n"
             set frm($tag) [join $lines \n]
-#puts "prefix after:$flds($tag)"
+	    #puts "prefix after:$flds($tag)"
             puts -nonewline $fout $frm($tag)
         } {X-GNATS-Notify} {
             #
             # Taken care of above with the mail header
             #
-            unset still_left($tag)
+	    if {[info exists still_left($tag)]} {
+		unset still_left($tag)
+	    }
         } {>Unformatted} {
             #
             # Taken care of later in the function...
@@ -470,6 +479,8 @@ proc write_pr {fout} {
             case $tag $Tkeditpr(singletextflds) {
                 #puts $fout "$tag:\t[string trim [textget $tag] "\t\n "]"
                 puts $fout "$tag:\t[string trim $frm($tag) "\t\n "]"
+	    } $Tkeditpr(shorttextflds) {
+                puts $fout "$tag:\t[string trim $frm($tag) "\t\n "]"
             } [concat >Category >Responsible >Submitter-Id $Tkeditpr(radioflds)] {
                 puts $fout "$tag:\t$frm($tag)"
             } $Tkeditpr(multitextflds) {
@@ -698,17 +709,36 @@ proc fillfrm {} {
 
     # listboxes and radio (enumerated) fields
     foreach tag [concat $Tkeditpr(listboxflds) $Tkeditpr(radioflds)] {
-        set  flds($tag) [string trim $flds($tag) "\t\n "]
+	if {[info exists flds($tag)]} {
+	    set flds($tag) [string trim $flds($tag) "\t\n "]
+	} {
+	    set flds($tag) ""
+	}
         #unset frm($tag)
         catch {set frm($tag) $flds($tag)}
     }
 
     # now the 1 line textual flds
     foreach tag $Tkeditpr(singletextflds) {
-        set flds($tag) [string trim $flds($tag) "\t\n "]
+	if {[info exists flds($tag)]} {
+	    set flds($tag) [string trim $flds($tag) "\t\n "]
+	} {
+	    set flds($tag) ""
+	}
         textset $tag $flds($tag)
     }
 
+    # now the 1 line textual flds
+    foreach tag $Tkeditpr(shorttextflds) {
+	if {[info exists flds($tag)]} {
+	    set flds($tag) [string trim $flds($tag) "\t\n "]
+	} {
+	    set flds($tag) ""
+	}
+	#puts stderr "Setting $tag to $flds($tag)"
+	textset $tag $flds($tag) .eboxs.shortones
+    }
+
     # now the multi line textual flds
     foreach tag $Tkeditpr(multitextflds) {
         if {[info exists flds($tag)]} {
@@ -736,7 +766,13 @@ proc reset_report {} {
 proc edit_email_originator {} {
     global Tkeditpr flds frm
     flush_singletext $Tkeditpr(singletextflds)
-    email_originator [ftrim $frm(X-GNATS-Notify)] \
+    flush_singletext $Tkeditpr(shorttextflds)
+
+    set gn ""
+    if {[info exists $frm(X-GNATS-Notify)]} {
+	set gn [ftrim $frm(X-GNATS-Notify)]
+    }
+    email_originator $gn \
             [ftrim $frm(>Responsible)] [ftrim $flds(Reply-To)] \
             [ftrim $frm(>Category)]/$Tkeditpr(prid) [ftrim $frm(>Synopsis)]
 }
@@ -776,12 +812,17 @@ proc edit_window {} {
         >Submitter-Id
         >Responsible
     }
+
+    set Tkeditpr(shorttextflds) {
+	>Release
+	>Cost
+	>XrefPR
+    }
     
     set Tkeditpr(singletextflds) {
         >Originator
-        X-GNATS-Notify
-        >Release
         >Synopsis
+	>IPsec-barf-location
     }
 
     if {$TkGnats(ReleaseBased)} {
@@ -792,9 +833,10 @@ proc edit_window {} {
         >Description
         >How-To-Repeat
         >Environment
-        >Organization
+	>IPsec-look
         >Fix
         >Audit-Trail
+        >Organization
         >Unformatted
         >Release-Note
     }
@@ -906,14 +948,19 @@ proc edit_window {} {
 
     frame .eflds
     radiobar_frame .eflds .eflds.lb
+    set panelnum 0
     foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
         if {[check_suppressed_field $efield] != 1} {
             bagged_radiobar .eflds.lb [string tolower $efield] $efield \
-                    $elist None frm $Tkeditpr(value_width)
+                    $elist None frm $Tkeditpr(value_width) $panelnum
+	    incr panelnum
         }
     }
     pack .eflds.lb -side left -pady 0
     pack .eflds    -side top  -pady 0 -fill x -anchor w
+
+    #puts stderr [concat "This is fun: " [grid slaves .eflds.lb]]
+
     
     frame .eboxs
     set   nboxes 1
@@ -926,10 +973,20 @@ proc edit_window {} {
             incr nboxes
         }
     }
+
+    frame .eboxs.shortones -relief groove -borderwidth 2
+    foreach f $Tkeditpr(shorttextflds) {
+        if {[check_suppressed_field $f] == 1} {
+            continue
+        }
+	lappend Tkeditpr(tlist) [singletext .eboxs.shortones $f 20 "" 14]
+    }
+    pack .eboxs.shortones -side left 
+
     if {$nboxes > 1} {
         pack .eboxs -side top  -anchor w -pady 2 -padx  0 -fill x
     }
-    
+
     foreach f $Tkeditpr(singletextflds) {
         if {[check_suppressed_field $f] == 1} {
             continue
