@@ -607,10 +607,6 @@ set help_text(Submitter-Id) {
      with the support organization should use the default value of `net'
      for this field.)
 }
-foreach submitter $TkGnats(SubmitterFile) {
-    append help_text(Submitter-Id) "\n    `[lindex [split $submitter ":"] 0]'\n          [lindex [split $submitter ":"] 1]\n"
-    append help_text(Submitter-Id) "\n          type:responseTime:contact:notify = [join [lrange [split $submitter ":"] 2 end] :]\n"
-}
 
 set help_text(Originator) {
 `>Originator:'
@@ -702,10 +698,6 @@ set help_text(Category) {
      problem lies.  The values for this field are defined by the Support
      Site.
 }
-foreach category $TkGnats(CategoryFile) {
-    append help_text(Category) "\n    `[lindex [split $category ":"] 0]'\n          [lindex [split $category ":"] 1]\n"
-    append help_text(Category) "\n          responsible:notify = [join [lrange [split $category ":"] 2 end] :]\n"
-}
 
 set help_text(Class) "\n`>Class:'\n     (ENUMERATED) The current class of the PR.  Accepted values are:\n"
 foreach class $TkGnats(ClassesFile) {
@@ -803,13 +795,6 @@ set help_text(Responsible) {
      (TEXT) The person responsible for this category.  GNATS retrieves
      this information from the `categories' file.
 }
-foreach responsible $TkGnats(ResponsibleFile) {
-    set email [lindex [split $responsible ":"] 2]
-    if {$email == ""} {
-        set email [lindex [split $responsible ":"] 0]
-    }
-    append help_text(Responsible) "\n    `[lindex [split $responsible ":"] 0]'\n          [lindex [split $responsible ":"] 1] (email: $email)\n"
-}
 
 set help_text(Arrival-Date) {
 `>Arrival-Date:'
