@@ -4,7 +4,7 @@ proc tksendpr_usage {{str ""}} {
     tk_dialog .tkgnatsError "TkGnats Error" \
             "${str}usage: tksendpr -server 'ServerInfo' \[-categories 'list'\] \
              \[-classes 'list'\] \[-states 'list'\] \
-             \[-submitters 'list'\] \[-responsible 'list'\]" "error" 0 "OK"
+            \[-db 'database'\] \[-submitters 'list'\] \[-responsible 'list'\]" "error" 0 "OK"
     exit
 }
 
@@ -25,9 +25,9 @@ proc tksendpr_process_args {} {
 	switch -exact -- $opt -server {
             set TkGnats(ServerInfo) $val
         } -categories {
-	    set TkGnats(CategoryFile)  $val
+	    set TkGnats(CategoryList)  $val
 	} -submitters {
-	    set TkGnats(SubmitterFile) $val
+	    set TkGnats(SubmitterList) $val
         } -responsible {
             set TkGnats(ResponsibleFile) $val
         } -classes {
@@ -533,14 +533,12 @@ pack    .action.msg -side left -fill x -expand 1
 
 frame .eflds
 radiobar_frame .eflds .eflds.lb
+set panelnum 0
 foreach {efield elist} [list Class $TkGnats(ClassesList) State $TkGnats(StatesList) Priority {low medium high} Severity {non-critical serious critical} Confidential {no yes}] {
     if {![check_suppressed_field $efield]} {
-        if {![info exists TkGnats(${efield}PerRow)]} {
-            set TkGnats(${efield}PerRow) [llength $elist]
-        }
-        radiobar     .eflds.lb [string tolower $efield] $efield $elist $TkGnats(${efield}PerRow) None
-        radiobar_set .eflds.lb [string tolower $efield] $TkGnats(CreateDefault$efield) \
-                $elist $TkGnats(${efield}PerRow)
+        radiobar     .eflds.lb [string tolower $efield] $efield $elist None "" "" 0 $panelnum
+        radiobar_set .eflds.lb [string tolower $efield] $TkGnats(CreateDefault$efield)
+	incr panelnum
     } {
         set $efield [lindex $elist 0]
     }
