@@ -214,6 +214,12 @@ proc view_action_bar {prid} {
 proc tkviewpr_formatted {prid} {
     global TkGnats Tkviewpr flds env current_multi_text
 
+    set Tkviewpr(shortflds) {
+	>Cost
+	>Release
+	>Responsible
+    }
+
     set Tkviewpr(radioflds) {
         >State
         >Confidential
@@ -226,11 +232,12 @@ proc tkviewpr_formatted {prid} {
         >Description
         >How-To-Repeat
         >Environment
-        >Organization
+	>IPsec-look
         >Fix
         >Audit-Trail
         >Unformatted
         >Release-Note
+        >Organization
     }
     
     set current_multi_text ""
@@ -296,7 +303,7 @@ proc tkviewpr_formatted {prid} {
     
     view_action_bar $prid
 
-    set flist [list Arrival-Date Last-Modified Closed-Date Class State Priority Severity Confidential Category Submitter-Id Responsible Originator X-GNATS-Notify Release Synopsis]
+    set flist [list Arrival-Date Last-Modified Closed-Date Class State Priority Severity Confidential Category Submitter-Id Responsible Originator X-GNATS-Notify Release Synopsis IPsec-barf-location Cost]
     if {$TkGnats(ReleaseBased)} {
         lappend flist  Keywords $TkGnats(Quarter) Date-Required
     }
