@@ -47,7 +47,9 @@ copy_adm_entry (AdmEntry *ent)
   res->field = ent->field;
   for (x = 0; x < ent->fieldcount; x++)
     {
-      res->admFields[x] = xstrdup (ent->admFields[x]);
+      res->admFields[x] = (ent->admFields[x] == NULL)
+			    ? NULL
+			    : xstrdup (ent->admFields[x]);
     }
   return res;
 }
