@@ -263,6 +263,7 @@ read_server (FILE *outfp)
 	    }
 	  fprintf (outfp, "%s", recvline + i);
 	}
+      free (recvline);
     }
 } 
 
@@ -329,12 +330,20 @@ get_reply (FILE *outfp)
 		}
 	      else
 		{
-		  read_server (outfp);
+		  if (outfp != NULL)
+		    read_server (outfp);
+		  else
+		    {
+		      fprintf (stderr, "%s: unexpected %d received\n",
+			       program_name, r->state);
+		      safe_exit();
+		    }
 		}
 	      break;
 
 	    case CODE_INFORMATION:
-	      fprintf (outfp, "%s\n", r->text);
+	      if (outfp != NULL)
+		fprintf (outfp, "%s\n", r->text);
 	      break;
 
 	    case CODE_NONEXISTENT_PR:
@@ -1216,7 +1225,7 @@ netSetEditEmailAddr (const char *addr)
 }
 
 static void
-netSendPRCmd (const char *cmd, FILE *file)
+netSendPRCmd (const char *cmd, FILE *file, int show_information)
 {
   char *line;
   if (debug)
@@ -1233,13 +1242,13 @@ netSendPRCmd (const char *cmd, FILE *file)
     }
   fprintf (serv_write, ".\r\n");
   /* if get_reply finds errors it writes messages and doesn't return */
-  get_reply (stdout);
+  get_reply (show_information ? stdout : NULL);
 }
 
 void
 netCheckPR (FILE *file, int initial)
 {
-  netSendPRCmd (initial ? "CHEK INIT" : "CHEK", file);
+  netSendPRCmd (initial ? "CHEK INIT" : "CHEK", file, 1);
 }
 
 void
@@ -1298,9 +1307,9 @@ netEditField (FILE *fp, const char *prnum, const char *fieldname,
 }
 
 void
-netSubmitNewPR (FILE *file)
+netSubmitNewPR (FILE *file, int show_prnum)
 {
-  netSendPRCmd ("SUBM", file);
+  netSendPRCmd ("SUBM", file, show_prnum);
 }
 
 void
@@ -1310,7 +1319,7 @@ netModifyPR (FILE *file, const char *prNum, const char *editEmailAddr)
 
   netSetEditEmailAddr (editEmailAddr);
   asprintf (&buf, "EDIT %s", prNum);
-  netSendPRCmd (buf, file);
+  netSendPRCmd (buf, file, 1);
   free (buf);
 }
 
