@@ -478,6 +478,7 @@ GNATS_subm (int ac, char **av ATTRIBUTE_UNUSED)
   char *tempfile;
   FILE *fp;
   ErrorDesc err;
+  int new_pr_num;
 
   if (ac != 0)
     {
@@ -507,9 +508,11 @@ GNATS_subm (int ac, char **av ATTRIBUTE_UNUSED)
     {
       if (daemon_lock_gnats (FALSE) == 0)
 	{
-	  if (submit_pr (currentDatabase, fp, &err) != 0)
+	  if ((new_pr_num = submit_pr (currentDatabase, fp, &err)) != 0)
 	    {
-	      printf ("%d PR added.\r\n", CODE_OK);
+	      printf ("%d-The added PR number is:\r\n",
+		      CODE_INFORMATION_FILLER);
+	      printf ("%d %d\r\n", CODE_INFORMATION, new_pr_num);
 	      fflush (stdout);
 	    }
 	  else
