@@ -39,6 +39,7 @@ struct databaseInfo
   char *databaseName;
   AdmEntry *databaseEnt;
   ChangeActions changeActions;
+  FieldList requiredInputFields;
   DatabaseFieldInfo fieldInfo;
   AdmEntry *hostList;
   IndexDesc indexDesc;
@@ -83,6 +84,7 @@ newDatabaseInfo (void)
   res->databaseName = NULL;
   res->databaseEnt = NULL;
   res->changeActions = NULL;
+  res->requiredInputFields = NULL;
   res->fieldInfo = NULL;
   res->hostList = NULL;
   res->indexDesc = NULL;
@@ -263,6 +265,15 @@ addGlobalChangeActions (DatabaseInfo database, ChangeActions actions)
     }
 }
 
+void
+setRequiredInputFields (DatabaseInfo database, FieldList list)
+{
+  if (databaseValid (database))
+    {
+      database->requiredInputFields = list;
+    }
+}
+
 void
 setIndexDesc (DatabaseInfo database, IndexDesc new)
 {
@@ -588,7 +599,7 @@ readAdmFile (const char *filename, int fields, AdmEntry **dest,
 
   while ((line = read_line (f, NULL)) != NULL)
     {
-      if (line[0] != '#' && line[0] != '\n' && !isspace ((int) line[0]))
+      if (line[0] != '#' && line[0] != '\n' && !isspace ((int)(unsigned char) line[0]))
 	{
 	  AdmEntry *nent = build_adm_entry (line, InvalidFieldIndex);
 
@@ -850,6 +861,12 @@ getAuditTrailFormat (const DatabaseInfo database)
     }
 }
 
+FieldList
+getRequiredInputFields (const DatabaseInfo database)
+{
+  return database->requiredInputFields;
+}
+
 IndexDesc
 getIndexDesc (const DatabaseInfo database)
 {
@@ -957,6 +974,7 @@ freeDatabaseInfo (DatabaseInfo database)
       clearHostList (database);
       clearInputTemplate (database);
       clearGlobalChangeActions (database);
+      freeFieldList (database->requiredInputFields);
       if (database->databaseName != NULL)
 	{
 	  free (database->databaseName);
