@@ -1,6 +1,6 @@
 #!/bin/sh
 # Program to edit problem reports for GNATS.
-# Copyright (C) 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001 Free Software
+# Copyright (C) 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002 Free Software
 # Foundation, Inc.
 # Contributed by Jeffrey Osier (jeffrey@cygnus.com).
 # Majorly revised by Bob Manson (manson@juniper.net).
@@ -114,12 +114,12 @@ PR_ADDR="$QUERY_PR --responsible-address"
 PR_EDIT="$LIBEXECDIR/gnats/pr-edit $GNATS_HOST $GNATS_PORT $EDIT_USER $EDIT_PASSWD $GNATS_DB"
 
 # These traps take care of deleting all the /tmp files
-trap 'rm -f $new $new.old $change_msg $fixfil ; exit 0' 0
+trap 'rm -f $new.old $change_msg $fixfil' 0
+# Don't delete $new on error signals
 trap 'if [ "$locked" != "" ]; then \
         $PR_EDIT --unlock $pr_id ; \
 	locked= ; \
-      fi ; \
-      rm -f $new $new.old $change_msg $fixfil ; exit 1' 1 2 3 13 15
+      fi' 1 2 3 13 15
 
 # find a user name
 if [ "$USER" != "" ]; then
@@ -296,4 +296,6 @@ while true; do
     done
 done
 
+rm -f $new
+
 exit 0
