@@ -24,38 +24,113 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "query.h"
 #include "mail.h"
 
-static int applyChangeActions (PR *pr, PR *oldPR, FieldIndex field, 
-			       ChangeActions actions, ErrorDesc *err,
-			       FormatNamedParameter *params);
-
-static int processFieldChange (PR *pr, PR *oldPR, FieldIndex field,
-			       ErrorDesc *err, 
-			       const char *editUserEmailAddr,
-			       const char *oldValue, const char *newValue);
-
 /* Euuugh.  There's gotta be a better way to keep this around.  */
 static char *newAuditTrailEntries = NULL;
 
-static void sendAuditMail (PR *pr, PR *oldPR, const char *editUserEmailAddr,
-			   const char *newAuditTrailEntries,
-			   ErrorDesc *err);
-
-int
-replace_pr (const DatabaseInfo database, FILE *fp,
-	    const char *editUserEmailAddr, ErrorDesc *err)
+static void
+sendAuditMail (PR *pr, PR *oldPR, const char *editUserEmailAddr, 
+	       const char *newAuditTrailEntries, ErrorDesc *err)
 {
-  PR *pr = allocPR (database);
+  FormatNamedParameter *parms = NULL;
+  const DatabaseInfo database = pr->database;
 
-  /* Read the message header in.  */
-  if (read_header (pr, fp) < 0)
+  parms = allocateNamedParameter ("$EditUserEmailAddr", editUserEmailAddr,
+				  parms);
+  parms = allocateNamedParameter ("$NewAuditTrail", newAuditTrailEntries,
+				  parms);
+
+  if (strcmp (field_value (pr, RESPONSIBLE (database)),
+	      field_value (oldPR, RESPONSIBLE (database))) != 0)
     {
-      setError (err, CODE_EOF_PR, "Couldn't read PR header");
-      return 0;
+      parms
+	= allocateNamedParameter ("$OldResponsible",
+				  field_value (oldPR, RESPONSIBLE (database)),
+				  parms);
     }
 
-  read_pr (pr, fp, 0);
+  composeMailMessage (pr, oldPR, "audit-mail", parms, NULL, err);
+  freeFormatParameterList (parms);
+}
 
-  return rewrite_pr (pr, editUserEmailAddr, err);
+/* Add an entry to the AUDIT_TRAIL field. PARAMS are the format
+   parameters used when composing the entry.  FMT is the format of the
+   audit-trail entry to add.  */
+
+static int
+addAuditTrailEnt (PR *pr, QueryFormat *fmt, 
+		  FormatNamedParameter *params, ErrorDesc *err)
+{
+  char *newAuditString = NULL;
+  char *finalAuditString;
+  const char *t;
+  char *currDate = get_curr_date ();
+
+  if (fmt == NULL)
+    {
+      fmt = getAuditTrailFormat (pr->database);
+    }
+
+  /* Format the new audit entry. */
+  process_format (NULL, &newAuditString, pr, NULL, fmt, "\n", params);
+
+  /* Squirrel it away, because we'll need to mail it later.  */
+  append_string (&newAuditTrailEntries, newAuditString);
+
+  /* Now append the formatted string to the audit-trail field.  */
+  t = field_value (pr, AUDIT_TRAIL (pr->database));
+  if (t == NULL)
+    {
+      t = "";
+    }
+  finalAuditString = xstrdup (t);
+  append_string (&finalAuditString, newAuditString);
+  set_field (pr, AUDIT_TRAIL (pr->database), finalAuditString, err);
+
+  free (finalAuditString);
+  free (newAuditString);
+  free (currDate);
+
+  return 0;
+}
+
+static int
+applyChangeAction (ChangeActions action, PR *pr, PR *oldPR, FieldIndex field,
+		   ErrorDesc *err, FormatNamedParameter *params)
+{
+  FieldEdit *fieldEdits = action->edits;
+  FieldList fields = action->requiredFields;
+
+  while (fields != NULL)
+    {
+      const char *fldval = get_field_value (pr, oldPR, fields->ent, NULL, NULL);
+      if (value_is_empty (fldval))
+	{
+	  setError (err, CODE_INVALID_PR_CONTENTS,
+		    "Required field %s missing from PR %s.",
+		    complexFieldIndexToString (fields->ent),
+		    field_value (pr, NUMBER (pr->database)));
+	  return 1;
+	}
+      fields = fields->next;
+    }
+
+  if (action->requireChangeReason && field_change_reason (pr, field) == NULL)
+    {
+      setError (err, CODE_INVALID_PR_CONTENTS,
+		"Edit of field %s requires a change reason.",
+		fieldDefForIndex (field)->name);
+      return 1;
+    }
+
+  while (fieldEdits != NULL)
+    {
+      if (applyFieldEdit (pr, fieldEdits, err, params) != 0)
+	{
+	  return 1;
+	}
+      fieldEdits = fieldEdits->next;
+    }
+  return 0;
 }
 
 static int
@@ -87,6 +162,110 @@ addAuditEntryP (const DatabaseInfo database, FieldIndex field,
   return 0;
 }
 
+static int
+applyChangeActions (PR *pr, PR *oldPR, FieldIndex field, 
+		    ChangeActions actions, ErrorDesc *err,
+		    FormatNamedParameter *params)
+{
+  {
+    ChangeActions actionList = actions;
+
+    while (actionList != NULL)
+      {
+	if (actionList->expr == NULL
+	    || pr_matches_expr (pr, oldPR, actionList->expr, params))
+	  {
+	    if (applyChangeAction (actionList, pr, oldPR, field, err, params))
+	      {
+		return 1;
+	      }
+	  }
+	actionList = actionList->next;
+      }
+  }
+
+  if (field != InvalidFieldIndex && addAuditEntryP (pr->database, 
+						    field, actions))
+    {
+      ChangeActions action = actions;
+      while (actions != NULL)
+	{
+	  if (actions->addAuditTrail)
+	    {
+	      break;
+	    }
+	  actions = actions->next;
+	}
+
+      if (action != NULL)
+	{
+	  addAuditTrailEnt (pr, action->auditTrailFormat, params, err);
+	}
+      else
+	{
+	  addAuditTrailEnt (pr, NULL, params, err);
+	}
+    }
+
+  return 0;
+}
+
+static int
+processFieldChange (PR *pr, PR *oldPR, FieldIndex field,
+		    ErrorDesc *err, const char *editUserEmailAddr,
+		    const char *oldValue, const char *newValue)
+{
+  ChangeActions actions;
+  FormatNamedParameter *params = NULL;
+  int res;
+
+  if (oldValue == NULL)
+    {
+      oldValue = "";
+    }
+  if (newValue == NULL)
+    {
+      newValue = "";
+    }
+
+  if (fieldDefForIndex (field)->readonly)
+    {
+      setError (err, CODE_READONLY_FIELD, 
+		newBadFieldEntry (field, NULL, NULL),
+		"Field %s is read-only: `%s'->`%s'",
+		fieldDefForIndex (field)->name,
+		oldValue,
+		newValue);
+      return 1;
+    }
+
+  {
+    char *currDate = get_curr_date ();
+    params = allocateNamedParameter ("$CurrentDate", currDate, params);
+    free (currDate);
+  }
+
+  params = allocateNamedParameter ("$FieldName", 
+				   fieldDefForIndex (field)->name, params);
+  params = allocateNamedParameter ("$OldValue", oldValue, params);
+  params = allocateNamedParameter ("$NewValue", newValue, params);
+  params = allocateNamedParameter ("$EditUserEmailAddr", editUserEmailAddr,
+				   params);
+  {
+    const char *reason = field_change_reason (pr, field);
+    if (reason == NULL)
+      {
+	reason = "";
+      }
+    params = allocateNamedParameter ("$ChangeReason", reason , params);
+  }
+
+  actions = fieldDefForIndex (field)->changeActions;
+  res = applyChangeActions (pr, oldPR, field, actions, err, params);
+  freeFormatParameterList (params);
+  return res;
+}
+
 static int
 processPRChanges (const char *editUserEmailAddr, PR *old_pr, PR *new_pr,
 		  ErrorDesc *err)
@@ -152,7 +331,14 @@ processPRChanges (const char *editUserEmailAddr, PR *old_pr, PR *new_pr,
 	  if (fieldDefForIndex (field)->readonly)
 	    {
 	      fieldsChanged[x] = 0;
-	      set_field (new_pr, field, old_value, err);
+	      if (old_value == NULL)
+	        {
+		  unsetField (new_pr, field);
+	        }
+	      else
+	        {
+		  set_field (new_pr, field, old_value, err);
+	        }
 	    }
 	  else
 	    {
@@ -211,85 +397,42 @@ processPRChanges (const char *editUserEmailAddr, PR *old_pr, PR *new_pr,
 /* Replace an existing PR with the same PR number as NEW_PR.
    EDITUSEREMAILADDR is the email address of the user that is performing the
    edit.  */
-int
-rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
+static int
+rewrite_pr (PR *pr, PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
 {
   const DatabaseInfo database = new_pr->database;
   FILE *prfile;
 
-  char *path =      NULL;
-  char *old_path =  NULL;
+  char *new_path = NULL;
+  char *old_path = NULL;
   char *bkup_path = NULL;
 
-  PR *old_pr = NULL;
-
   int res = 1;
 
   newAuditTrailEntries = NULL;
 
-  if (field_value (new_pr, NUMBER (database)) == NULL)
-    {
-      free_pr (new_pr);
-      setError (err, CODE_INVALID_PR_CONTENTS,
-		"Missing PR number from new PR contents");
-      return 0;
-    }
-
   if (field_value (new_pr, CATEGORY (database)) == NULL)
     {
-      free_pr (new_pr);
       setError (err, CODE_INVALID_PR_CONTENTS,
 		"Missing category from new PR contents");
       return 0;
     }
 
-  if (! prExists (new_pr->database, field_value (new_pr, NUMBER (database)),
-		  err))
-    {
-      free_pr (new_pr);
-      return 0;
-    }
-
   if (! isPrLocked (new_pr->database, field_value (new_pr, NUMBER (database))))
     {
       setError (err, CODE_PR_NOT_LOCKED, "PR %s is not locked", 
 		field_value (new_pr, NUMBER (database)));
-      free_pr (new_pr);
       return 0;
     }
 
-  if (! check_pr (new_pr, err, 0))
+  if (processPRChanges (editUserEmailAddr, pr, new_pr, err) != 0)
     {
       return 0;
     }
 
-  old_pr = replaceCurrentPRInIndex (new_pr);
-
-  if (old_pr == NULL)
-    {
-      setError (err, CODE_NO_INDEX, "Unable to locate existing PR %s",
-		field_value (new_pr, NUMBER (database)));
-      free_pr (new_pr);
-      return 0;
-    }
-
-  old_path = gen_pr_path (old_pr);
-
-  /* set this to be the file to be saved. now called .old. */
-  asprintf (&bkup_path, "%s.old", old_path);
-
-  if (processPRChanges (editUserEmailAddr, old_pr, new_pr, err) != 0)
-    {
-      clearPRChain (database);
-      free_pr (old_pr);
-      free (bkup_path);
-      free (old_path);
-      return 0;
-    }
-
   /* check to see if the category changes, and if so, write the 
      new file out, and unlink the old one.  */
-  if (strcmp (field_value (old_pr, CATEGORY (database)),
+  if (strcmp (field_value (pr, CATEGORY (database)),
 	      field_value (new_pr, CATEGORY (database))) != 0)
     {
       char *dirPath;
@@ -309,10 +452,6 @@ rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
 			    "Error creating directory for category %s: %s",
 			    field_value (new_pr, CATEGORY (database)),
 			    strerror (errno));
-		  clearPRChain (database);
-		  free_pr (old_pr);
-		  free (bkup_path);
-		  free (old_path);
 		  free (dirPath);
 		  return 0;
 		}
@@ -322,44 +461,32 @@ rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
 	      setError (err, CODE_FILE_ERROR, 
 			"Directory for category %s does not exist",
 			field_value (new_pr, CATEGORY (database)));
-	      clearPRChain (database);
-	      free_pr (old_pr);
-	      free (bkup_path);
-	      free (old_path);
 	      free (dirPath);
 	      return 0;
 	    }
 	}
       free (dirPath);
-      path = gen_pr_path (new_pr);
-    }
-  else
-    {
-      path = gen_pr_path (old_pr);
     }
 
+  /* backup the current PR file */
+  old_path = gen_pr_path (pr);
+  asprintf (&bkup_path, "%s.old", old_path);
   if (rename (old_path, bkup_path) < 0)
     {
       if (errno != EXDEV)
 	{
 	  setError (err, CODE_FILE_ERROR, "Could not rename %s: %s",
 		    old_path, strerror (errno));
-	  clearPRChain (database);
-	  free_pr (old_pr);
 	  free (bkup_path);
 	  free (old_path);
-	  free (path);
 	  return 0;
 	}
       if (copy_file (old_path, bkup_path) != 0)
 	{
 	  setError (err, CODE_FILE_ERROR, "Could not copy %s to %s: %s",
 		    old_path, bkup_path, strerror (errno));
-	  clearPRChain (database);
-	  free_pr (old_pr);
 	  free (bkup_path);
 	  free (old_path);
-	  free (path);
 	  return 0;
 	}
 
@@ -369,16 +496,15 @@ rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
     }
 
   /* Now build the file.  */
-  prfile = fopen (path, "w+");
+  new_path = gen_pr_path (new_pr);
+  prfile = fopen (new_path, "w+");
   if (prfile == (FILE *) NULL)
     {
       setError (err, CODE_FILE_ERROR, "Cannot write the PR to %s: %s",
-		path, strerror (errno));
-      clearPRChain (database);
-      free_pr (old_pr);
+		new_path, strerror (errno));
       free (bkup_path);
       free (old_path);
-      free (path);
+      free (new_path);
       return 0;
     }
 
@@ -394,25 +520,28 @@ rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
   if (fclose (prfile) == EOF)
     {
       setError (err, CODE_FILE_ERROR, "Error writing out PR %s: %s",
-		path, strerror (errno));
-      clearPRChain (database);
-      free_pr (old_pr);
+		new_path, strerror (errno));
       free (bkup_path);
       free (old_path);
-      free (path);
+      free (new_path);
       return 0;
     }
 
   unlink (bkup_path);
 
   /* unlink the old file, if it is in another category dir. */
-  if (strcmp (field_value (old_pr, CATEGORY (database)),
+  if (strcmp (field_value (pr, CATEGORY (database)),
 	      field_value (new_pr, CATEGORY (database))) != 0)
     {
       unlink (old_path);
     }
 
+  free (bkup_path);
+  free (old_path);
+  free (new_path);
+
   /* write out the new index.  */
+  replaceCurrentPRInIndex (pr, new_pr, err);
   if (writeIndex (database, err) != 0)
     {
       res = 0;
@@ -421,16 +550,64 @@ rewrite_pr (PR *new_pr, const char *editUserEmailAddr, ErrorDesc *err)
   /* Send mail about the edit.  */
   if (newAuditTrailEntries != NULL)
     {
-      sendAuditMail (new_pr, old_pr, editUserEmailAddr, newAuditTrailEntries,
+      sendAuditMail (new_pr, pr, editUserEmailAddr, newAuditTrailEntries,
 		     err);
       free (newAuditTrailEntries);
       newAuditTrailEntries = NULL;
     }
 
-  free_pr (old_pr);
-  free (bkup_path);
-  free (old_path);
-  free (path);
+  return res;
+}
+
+
+int
+replace_pr (const DatabaseInfo database, FILE *fp,
+	    const char *editUserEmailAddr, ErrorDesc *err)
+{
+  int res;
+  const char *prnum = NULL;
+  PR *curr_pr;
+  PR *new_pr = allocPR (database);
+
+  /* Read the message header in.  */
+  if (read_header (new_pr, fp) < 0)
+    {
+      setError (err, CODE_EOF_PR, "Couldn't read PR header");
+      return 0;
+    }
+
+  read_pr (new_pr, fp, 0);
+
+  prnum = field_value (new_pr, NUMBER (database));
+  if (prnum == NULL)
+    {
+      setError (err, CODE_INVALID_PR_CONTENTS,
+	        "Missing PR number from new PR contents");
+      return 0;
+    }
+
+  if (! prExists (database, prnum, err))
+    {
+      return 0;
+    }
+
+  curr_pr = readPRWithNum (database, prnum, 0, err);
+  if (curr_pr == NULL)
+    {
+      setError (err, CODE_NO_INDEX,
+	        "Unable to locate existing PR %s", prnum);
+      return 0;
+    }
+
+  if (! check_pr (new_pr, err, 0))
+    {
+      return 0;
+    }
+
+  res = rewrite_pr (curr_pr, new_pr, editUserEmailAddr, err);
+
+  free_pr (curr_pr);
+  free_pr (new_pr);
 
   return res;
 }
@@ -815,17 +992,25 @@ edit_field (const DatabaseInfo database, const char *prnum,
   char pid[32];
   const char *oldfield;
   int res;
-  PR *pr;
+  PR *pr, *new_pr;
 
   if (! prExists (database, prnum, err))
     {
       free (newcontents);
+      if (changeReason != NULL)
+        {
+	  free (changeReason);
+        }
       return 0;
     }
   if (fieldIndex == InvalidFieldIndex)
     {
       setError (err, CODE_INVALID_FIELD_NAME, "Invalid field name");
       free (newcontents);
+      if (changeReason != NULL)
+        {
+	  free (changeReason);
+        }
       return 0;
     }
   if (requiresChangeReason (fieldDefForIndex (fieldIndex))
@@ -837,10 +1022,15 @@ edit_field (const DatabaseInfo database, const char *prnum,
       free (newcontents);
       return 0;
     }
+
   sprintf (pid, "%d", (int) getpid ());
   if (lock_pr (database, prnum, "edit_field", pid, err) == 0)
     {
       free (newcontents);
+      if (changeReason != NULL)
+        {
+	  free (changeReason);
+        }
       return 0;
     }
 
@@ -849,7 +1039,6 @@ edit_field (const DatabaseInfo database, const char *prnum,
     {
       unlock_pr (database, prnum, err);
       setError (err, CODE_UNREADABLE_PR, "Error reading PR %s.", prnum);
-      free_pr (pr);
       free (newcontents);
       if (changeReason != NULL)
 	{
@@ -876,23 +1065,40 @@ edit_field (const DatabaseInfo database, const char *prnum,
       newcontents = totalVal;
     }
 
+  /* create a copy of the currently active pr so that we can modify it and
+     do subsequent validation allowing for us to bail out cleanly */
+  new_pr = allocPR (database);
+  set_field (new_pr, NUMBER (database), prnum, err);
+  set_field (new_pr, CATEGORY (database), field_value (pr, CATEGORY (database)),
+	     err);
+  fillInPR (new_pr, err);
+
   /* set_field () verifies that the value is valid before doing the
      set.  */
-  if (set_field (pr, fieldIndex, newcontents, err) == 0)
+  /* set_field returns a boolean!!! */
+  if (set_field (new_pr, fieldIndex, newcontents, err) == 0)
     {
       res = 0;
-      free_pr (pr);
     }
   else
     {
       if (changeReason != NULL)
 	{
-	  setFieldChangeReason (pr, fieldIndex, changeReason);
+	  setFieldChangeReason (new_pr, fieldIndex, changeReason);
 	  free (changeReason);
 	  changeReason = NULL;
 	}
 
-      res = rewrite_pr (pr, editUserEmailAddr, err);
+      res = rewrite_pr (pr, new_pr, editUserEmailAddr, err);
+    }
+
+  free_pr (pr);
+  free_pr (new_pr);
+  free (newcontents);
+  if (changeReason != NULL)
+    {
+      free (changeReason);
+      changeReason = NULL;
     }
 
   if (unlock_pr (database, prnum, err) == 0)
@@ -900,12 +1106,6 @@ edit_field (const DatabaseInfo database, const char *prnum,
       res = 0;
     }
 
-  free (newcontents);
-  if (changeReason != NULL)
-    {
-      free (changeReason);
-      changeReason = NULL;
-    }
   return res;
 }
 
@@ -1029,261 +1229,32 @@ applyFieldEdit (PR *pr, FieldEdit *edit, ErrorDesc *err,
   return res;
 }
 
-/* Add an entry to the AUDIT_TRAIL field. PARAMS are the format
-   parameters used when composing the entry.  FMT is the format of the
-   audit-trail entry to add.  */
-
-static int
-addAuditTrailEnt (PR *pr, QueryFormat *fmt, 
-		  FormatNamedParameter *params, ErrorDesc *err)
-{
-  char *newAuditString = NULL;
-  char *finalAuditString;
-  const char *t;
-  char *currDate = get_curr_date ();
-
-  if (fmt == NULL)
-    {
-      fmt = getAuditTrailFormat (pr->database);
-    }
-
-  /* Format the new audit entry. */
-  process_format (NULL, &newAuditString, pr, NULL, fmt, "\n", params);
-
-  /* Squirrel it away, because we'll need to mail it later.  */
-  append_string (&newAuditTrailEntries, newAuditString);
-
-  /* Now append the formatted string to the audit-trail field.  */
-  t = field_value (pr, AUDIT_TRAIL (pr->database));
-  if (t == NULL)
-    {
-      t = "";
-    }
-  finalAuditString = xstrdup (t);
-  append_string (&finalAuditString, newAuditString);
-  set_field (pr, AUDIT_TRAIL (pr->database), finalAuditString, err);
-
-  free (finalAuditString);
-  free (newAuditString);
-  free (currDate);
-
-  return 0;
-}
-
-static void
-sendAuditMail (PR *pr, PR *oldPR, const char *editUserEmailAddr, 
-	       const char *newAuditTrailEntries, ErrorDesc *err)
-{
-  FormatNamedParameter *parms = NULL;
-  const DatabaseInfo database = pr->database;
-
-  parms = allocateNamedParameter ("$EditUserEmailAddr", editUserEmailAddr,
-				  parms);
-  parms = allocateNamedParameter ("$NewAuditTrail", newAuditTrailEntries,
-				  parms);
-
-  if (strcmp (field_value (pr, RESPONSIBLE (database)),
-	      field_value (oldPR, RESPONSIBLE (database))) != 0)
-    {
-      parms
-	= allocateNamedParameter ("$OldResponsible",
-				  field_value (oldPR, RESPONSIBLE (database)),
-				  parms);
-    }
-
-  composeMailMessage (pr, oldPR, "audit-mail", parms, NULL, err);
-  freeFormatParameterList (parms);
-}
-
-static int
-applyChangeAction (ChangeActions action, PR *pr, PR *oldPR, FieldIndex field,
-		   ErrorDesc *err, FormatNamedParameter *params)
-{
-  FieldEdit *fieldEdits = action->edits;
-  FieldList fields = action->requiredFields;
-
-  while (fields != NULL)
-    {
-      const char *fldval = get_field_value (pr, oldPR, fields->ent, NULL, NULL);
-      if (value_is_empty (fldval))
-	{
-	  setError (err, CODE_INVALID_PR_CONTENTS,
-		    "Required field %s missing from PR %s.",
-		    complexFieldIndexToString (fields->ent),
-		    field_value (pr, NUMBER (pr->database)));
-	  return 1;
-	}
-      fields = fields->next;
-    }
-
-  if (action->requireChangeReason && field_change_reason (pr, field) == NULL)
-    {
-      setError (err, CODE_INVALID_PR_CONTENTS,
-		"Edit of field %s requires a change reason.",
-		fieldDefForIndex (field)->name);
-      return 1;
-    }
-
-  while (fieldEdits != NULL)
-    {
-      if (applyFieldEdit (pr, fieldEdits, err, params) != 0)
-	{
-	  return 1;
-	}
-      fieldEdits = fieldEdits->next;
-    }
-  return 0;
-}
-
-static int
-applyChangeActions (PR *pr, PR *oldPR, FieldIndex field, 
-		    ChangeActions actions, ErrorDesc *err,
-		    FormatNamedParameter *params)
-{
-  {
-    ChangeActions actionList = actions;
-
-    while (actionList != NULL)
-      {
-	if (actionList->expr == NULL
-	    || pr_matches_expr (pr, oldPR, actionList->expr, params))
-	  {
-	    if (applyChangeAction (actionList, pr, oldPR, field, err, params))
-	      {
-		return 1;
-	      }
-	  }
-	actionList = actionList->next;
-      }
-  }
-
-  if (field != InvalidFieldIndex && addAuditEntryP (pr->database, 
-						    field, actions))
-    {
-      ChangeActions action = actions;
-      while (actions != NULL)
-	{
-	  if (actions->addAuditTrail)
-	    {
-	      break;
-	    }
-	  actions = actions->next;
-	}
-
-      if (action != NULL)
-	{
-	  addAuditTrailEnt (pr, action->auditTrailFormat, params, err);
-	}
-      else
-	{
-	  addAuditTrailEnt (pr, NULL, params, err);
-	}
-    }
-
-  return 0;
-}
-
-static int
-processFieldChange (PR *pr, PR *oldPR, FieldIndex field,
-		    ErrorDesc *err, const char *editUserEmailAddr,
-		    const char *oldValue, const char *newValue)
-{
-  ChangeActions actions;
-  FormatNamedParameter *params = NULL;
-  int res;
-
-  if (oldValue == NULL)
-    {
-      oldValue = "";
-    }
-  if (newValue == NULL)
-    {
-      newValue = "";
-    }
-
-  if (fieldDefForIndex (field)->readonly)
-    {
-      setError (err, CODE_READONLY_FIELD, 
-		newBadFieldEntry (field, NULL, NULL),
-		"Field %s is read-only: `%s'->`%s'",
-		fieldDefForIndex (field)->name,
-		oldValue,
-		newValue);
-      return 1;
-    }
-
-  {
-    char *currDate = get_curr_date ();
-    params = allocateNamedParameter ("$CurrentDate", currDate, params);
-    free (currDate);
-  }
-
-  params = allocateNamedParameter ("$FieldName", 
-				   fieldDefForIndex (field)->name, params);
-  params = allocateNamedParameter ("$OldValue", oldValue, params);
-  params = allocateNamedParameter ("$NewValue", newValue, params);
-  params = allocateNamedParameter ("$EditUserEmailAddr", editUserEmailAddr,
-				   params);
-  {
-    const char *reason = field_change_reason (pr, field);
-    if (reason == NULL)
-      {
-	reason = "";
-      }
-    params = allocateNamedParameter ("$ChangeReason", reason , params);
-  }
-
-  actions = fieldDefForIndex (field)->changeActions;
-  res = applyChangeActions (pr, oldPR, field, actions, err, params);
-  freeFormatParameterList (params);
-  return res;
-}
-
 /* Delete PR PRNUM.  */
 int
 deletePR (const DatabaseInfo database, const char *prnum, 
 	  const char *editUserEmailAddr, ErrorDesc *err)
 {
-  char *path = lookup_pr_path (database, prnum, err);
   char pid[32];
   FormatNamedParameter *parms = NULL;
-
-  if (path == NULL)
-    {
-      return -1;
-    }
+  short delete_status;
 
   if (client_lock_gnats (database, err))
     {
-      free (path);
       return -4;
     }
 
   if (lock_pr (database, prnum, GNATS_USER, pid, err) == 0)
     {
       client_unlock_gnats ();
-      free (path);
       return -4;
     }
 
-  if (removePRFromIndex (database, prnum, err))
+  if ((delete_status = pr_delete (database, prnum, err)) < 0)
     {
       unlock_pr (database, prnum, err);
       client_unlock_gnats ();
-      free (path);
-      return -5;
+      return delete_status;
     }
-  
-  if (unlink (path))
-    {
-      setError (err, CODE_FILE_ERROR, "Unable to unlink file %s\n", path);
-      unlock_pr (database, prnum, err);
-      client_unlock_gnats ();
-      free (path);
-      return -6;
-    }
-
-  free (path);
 
   if (unlock_pr (database, prnum, err) == 0)
     {
