@@ -118,19 +118,20 @@ createNewPRFile (PR *pr, int flag_autocreate, ErrorDesc *err)
 	}
     }
 
-  if (submitter->admFields[SubmitterAdmRtime][0] != '\0')
+  if (submitter->admFields[SubmitterAdmRtime] == NULL ||
+      submitter->admFields[SubmitterAdmRtime][0] == '\0')
     {
-      submitter_rtime = atoi (submitter->admFields[SubmitterAdmRtime]);
+      submitter_rtime = -1;
     }
   else
     {
-      submitter_rtime = -1;
+      submitter_rtime = atoi (submitter->admFields[SubmitterAdmRtime]);
     }
 
   /* If originator wasn't set, try to get the value from the From mail
      header.  This is what users usually expect. */
   {
-    FieldIndex idx_origin = find_field_index (database, "Originator");
+    FieldIndex idx_origin = ORIGINATOR (database);
     const char *value_origin = (idx_origin == InvalidFieldIndex
 				? NULL
 				: field_value (pr, idx_origin));
@@ -919,11 +920,13 @@ getBugNumber (const DatabaseInfo database, ErrorDesc *err)
   return bug_number;
 }
 
-/* Submit the PR whose contents are referred to by FP.  */
+/* Submit the PR whose contents are referred to by FP.
+ * Return the new PR number if it's a new PR, or return the PR number of
+ * the PR which was appended to if it's an existing PR. */
 int
 submit_pr (const DatabaseInfo database, FILE *fp, ErrorDesc *err)
 {
-  int result;
+  int result, retval;
   PR *pr = allocPR (database);
 
   result = (read_header (pr, fp) >= 0);
@@ -945,10 +948,8 @@ submit_pr (const DatabaseInfo database, FILE *fp, ErrorDesc *err)
 	    }
 	  else
 	    {
-	      if (append_report (fp, pr, prID, err) != 0)
-		{
-		  result = 0;
-		}
+	      retval = append_report (fp, pr, prID, err);
+	      result = (retval == 0) ? atoi (prID) : 0;
 	      free_pr (pr);
 	    }
 	  free (prID);
@@ -956,10 +957,8 @@ submit_pr (const DatabaseInfo database, FILE *fp, ErrorDesc *err)
       else
 	{
 	  read_pr (pr, fp, 0);
-	  if (createNewPRFile (pr, createCategoryDirs (database), err) < 0)
-	    {
-	      result = 0;
-	    }
+	  retval = createNewPRFile (pr, createCategoryDirs (database), err);
+	  result = (retval < 0) ? 0 : retval;
 	}
     }
   else
