@@ -19,15 +19,10 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "gnats.h"
 #ifdef HAVE_LIBCRYPT
-#ifdef HAVE_CRYPT		/* some systems declare `crypt' in unistd.h */
+#ifdef HAVE_CRYPT_H		/* some systems declare `crypt' in unistd.h */
 #include <crypt.h>
 #endif
 #endif
-#include <getopt.h>
-#include <stdio.h>
-#include <time.h>
-#include <unistd.h>
-
 
 enum _Crypt_Type { NONE, PLAIN, CRYPT, MD5 };
 typedef enum _Crypt_Type Crypt_Type;
@@ -36,7 +31,7 @@ typedef enum _Crypt_Type Crypt_Type;
 #define PROGRAM_NAME "gnats-pwconv"
 const char *program_name = PROGRAM_NAME;
 
-static const int MAXLINE = 1024;
+/* static const int MAXLINE = 1024; */
 static const char* const VERSION = "1.0";
 
 static const char* USAGE[] =
