@@ -32,19 +32,31 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 
 #include "config.h"
 
-/* Define these for the default settings */
-#include <ansidecl.h>
-#include <libiberty.h>
-
 #include <stdio.h>
 #include <time.h>
 /* On FreeBSD this ends up defining RE_DUP_MAX.  We need to make sure it's
    included before our regex.h. */
 #include <sys/param.h>
 
+/* Use some miscelaneous functions; should be included after stdio.h */
+#include "ansidecl.h"
+
+/* If system have getopt_long() it seems GNU compliant and likely use
+   GNU-compatible getopt() as well. Otherwise, we will use getopt*()
+   from libiberty. */
+#ifdef HAVE_GETOPT
+#define HAVE_DECL_GETOPT 1
+#else
+#define HAVE_DECL_GETOPT 0
+#endif /* HAVE_GETOPT */
+#ifdef HAVE_GETOPT_LONG
+#include <getopt.h>
+#else
+#include "gnugetopt.h"
+#endif /* HAVE_GETOPT_LONG */
+
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <getopt.h>
 #ifndef S_ISREG			/* Doesn't have POSIX.1 stat stuff. */
 #define mode_t unsigned short
 #endif
@@ -86,9 +98,11 @@ extern int errno;
 #include <sys/file.h>
 #endif
  
-#ifdef HAVE_LIBGEN_H
+#if defined(HAVE_LIBGEN_H)
+#  if defined(HAVE_BASENAME) || defined(HAVE_DIRNAME)
 #include <libgen.h>
-#endif
+#  endif
+#endif /* HAVE_LIBGEN_H */
 
 #ifndef alloca
 /* Make alloca work the best possible way.  */
@@ -292,7 +306,7 @@ extern void netEditField (FILE *fieldData, const char *prNum,
 			  const char *editUserEmailAddr, int appendToField,
 			  char *reason);
 
-extern void netSubmitNewPR (FILE *file);
+extern void netSubmitNewPR (FILE *file, int show_prnum);
 extern void netModifyPR (FILE *file, const char *prNum,
 			 const char *editUserEmailAddr);
 extern void netLockDB (void);
@@ -315,8 +329,6 @@ extern int check_pr (PR *pr,
 		     int initial_entry);
 extern int replace_pr (const DatabaseInfo database, FILE *infile, 
 		       const char *editUserEmailAddr, ErrorDesc *err);
-extern int rewrite_pr (PR *pr, const char *editUserEmailAddr, 
-		       ErrorDesc *err);
 extern int lock_pr (const DatabaseInfo database,
 		    const char *filename, const char *username,
 		    const char *processid, ErrorDesc *error);
@@ -394,6 +406,9 @@ extern FILE *open_mail_file (const DatabaseInfo database);
 extern void close_mail_file (FILE *);
 extern char *quote_string (const char *string);
 extern StringList *new_string_list_ent (char *string, StringList *next);
+extern PTR xmalloc (size_t size);
+extern PTR xrealloc (PTR ptr, size_t size);
+extern char *xstrdup (const char *string);
 extern char *xstrndup (const char *string, size_t length);
 extern const char *temporary_directory (void);
 extern int open_temporary_file (char *template, int mode);
@@ -404,5 +419,14 @@ extern size_t gnats_strftime (char *s, size_t size, const char *template,
 extern void usage (const char *const texts[], int exit_code);
 extern void version (const char *const program_name);
 extern bool value_is_empty (const char *string);
+#ifndef HAVE_ASPRINTF
+extern int asprintf (char **ret, const char *format, ...) ATTRIBUTE_PRINTF_2;
+#endif
+#ifndef HAVE_VASPRINTF
+extern int vasprintf (char **ret, const char *format, va_list) ATTRIBUTE_PRINTF(2,0);
+#endif
+#ifndef HAVE_BASENAME
+extern char *basename (char *path);
+#endif
 
 #endif /* !_gnats_h_ */
