@@ -24,7 +24,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111, USA.  */
 #include "query.h"
 
 #ifdef HAVE_LIBCRYPT
-#ifdef HAVE_CRYPT		/* some systems declare `crypt' in unistd.h */
+#ifdef HAVE_CRYPT_H		/* some systems declare `crypt' in unistd.h */
 #include <crypt.h>
 #endif
 #endif
@@ -478,7 +478,7 @@ findUserAccessLevel (const char *file, const char *user, const char *passwd,
 			 requested database. */
 		      const char *l2 = ent->admFields[3];
 		      
-		      if (! strlen(l2))
+		      if (l2 == NULL || ! strlen(l2))
 			found = 1;
 		      
 		      while (l2 != NULL && ! found)
