@@ -61,6 +61,7 @@ struct indexEntry
   char *buf;
   char **fields;
   PR *nextPR;
+  PR *prevPR;
 };
 
 static PR *nextIndexEntryBinary (IndexFileDesc fp);
@@ -234,6 +235,7 @@ nextIndexEntry (IndexFileDesc fp)
   p = res->index;
   seplen = strlen (fp->desc->separator);
 
+  p->prevPR = NULL;
   p->nextPR = NULL;
   p->fields = (char **) xmalloc (sizeof (char *) 
 				 * get_num_fields (fp->desc->database));
@@ -326,6 +328,7 @@ nextIndexEntryBinary (IndexFileDesc fp)
 
       res = allocPR (fp->desc->database);
       indexEnt = res->index;
+      indexEnt->prevPR = NULL;
       indexEnt->nextPR = NULL;
       indexEnt->fields 
 	= (char **) xmalloc (sizeof (char *) 
@@ -573,6 +576,7 @@ getIndex (IndexDesc indexDesc, ErrorDesc *err)
 	    }
 	  else
 	    {
+	      i->index->prevPR = end_chain;
 	      end_chain->index->nextPR = i;
 	      end_chain = i;
 	    }
@@ -662,6 +666,7 @@ allocIndex (PR *pr)
   pr->index = (Index *) xmalloc (sizeof (Index));
   pr->index->buf = NULL;
   pr->index->fields = NULL;
+  pr->index->prevPR = NULL;
   pr->index->nextPR = NULL;
 }
  
@@ -1115,34 +1120,57 @@ clearPRChain (const DatabaseInfo database)
     }
 }
 
-/* Find the PR in the current index with the same PR number as NEW_PR,
-   and replace it with NEW_PR.  The old PR is returned, or a NULL
-   pointer if a PR with the same number is not found.  */
+/* Replace a PR in the index with a copy of a new PR.
+   By generating a copy of the new PR, we allow for the new PR
+   to be free'd without harming the index pr chain. */
 
-PR *
-replaceCurrentPRInIndex (PR *new_pr)
+int
+replaceCurrentPRInIndex (PR *curr_pr, PR *new_pr, ErrorDesc *err)
 {
-  ErrorDesc err;
-  IndexDesc indexDesc = getIndexDesc (new_pr->database);
-  PR **currptr = &(indexDesc->prChain);
-  const char *new_pr_num = field_value (new_pr, NUMBER (new_pr->database));
+  PR *pr_replace, *pr_copy;
+  const DatabaseInfo database = curr_pr->database;
+  const char *new_pr_num = field_value (new_pr, NUMBER (database));
+  const char *curr_pr_num = field_value (curr_pr, NUMBER (database));
 
-  getFirstPR (new_pr->database, &err);
-
-  while ((*currptr) != NULL)
+  if (strcmp (curr_pr_num, new_pr_num) != 0)
     {
-      if (strcmp (field_value (*currptr, NUMBER (new_pr->database)),
-		  new_pr_num) == 0)
-	{
-	  PR *old_pr = (*currptr);
-	  new_pr->index->nextPR = old_pr->index->nextPR;
-	  old_pr->index->nextPR = NULL;
-	  (*currptr) = new_pr;
-	  return old_pr;
-	}
-      currptr = &((*currptr)->index->nextPR);
+      return 0;
     }
-  return NULL;
+
+  /* create a copy of the new pr to be put into the index pr chain */
+  pr_copy = allocPR (database);
+  set_field (pr_copy, NUMBER (curr_pr->database), new_pr_num, err);
+  set_field (pr_copy, CATEGORY (database),
+	     field_value (new_pr, CATEGORY (database)), err);
+  setPrevPR (pr_copy, curr_pr->index->prevPR);
+  setNextPR (pr_copy, curr_pr->index->nextPR);
+  fillInPR (pr_copy, err);
+
+  /* put the copy of new_pr into place in the index pr chain... */
+
+  if (curr_pr->index->prevPR == NULL)
+    {
+      /* the pr to be replaced is at the head of the pr chain */
+      IndexDesc indexDesc = getIndexDesc (database);
+      free_pr (indexDesc->prChain);
+      indexDesc->prChain = pr_copy;
+      return 1;
+    }
+
+  /* save a pointer to the pr to be replaced so that it can later be free'd */
+  pr_replace = getNextPR (curr_pr->index->prevPR);
+
+  setNextPR (curr_pr->index->prevPR, pr_copy);
+
+  if (curr_pr->index->nextPR != NULL)
+    {
+      /* the pr to be replaced is not at the end of the pr chain */
+      setPrevPR (curr_pr->index->nextPR, pr_copy);
+    }
+
+  free_pr (pr_replace);
+
+  return 1;
 }
 
 /* Remove PR PRNUM from the index for the current database, and rewrite the
@@ -1196,6 +1224,24 @@ getNextPR (PR *pr)
   return pr->index->nextPR;
 }
 
+void
+setNextPR (PR *pr, PR *next_pr)
+{
+  pr->index->nextPR = next_pr;
+}
+
+PR *
+getPrevPR (PR *pr)
+{
+  return pr->index->prevPR;
+}
+
+void
+setPrevPR (PR *pr, PR *prev_pr)
+{
+  pr->index->prevPR = prev_pr;
+}
+
 int
 indexIsBinary (const DatabaseInfo database)
 {
