@@ -25,4 +25,4 @@
 
 MAIL_AGENT="xDEFAULT_MAIL_AGENTx"
 
-exec $MAIL_AGENT $@
+exec $MAIL_AGENT $@ > /dev/null
