@@ -65,9 +65,13 @@ get_responsible_address (const DatabaseInfo database, const char *person)
      passwd file.  */
   if (res != NULL)
     {
-      if (res->admFields[ResponsibleAdmAlias] == '\0')
+      if (res->admFields[ResponsibleAdmAlias] == NULL ||
+	  res->admFields[ResponsibleAdmAlias] == '\0')
 	{
-	  free (res->admFields[ResponsibleAdmAlias]);
+	  if (res->admFields[ResponsibleAdmAlias] != NULL)
+	    {
+	      free (res->admFields[ResponsibleAdmAlias]);
+	    }
 	  res->admFields[ResponsibleAdmAlias] 
 	    = xstrdup (res->admFields[ResponsibleAdmKey]);
 	}
@@ -226,12 +230,15 @@ getOneAddress (const char **addrPtr)
   addrEnd = addr;
 
   /* ignore any ending white space */
-  while (addr >= addrStart && isspace((int)(unsigned char)*(addr-sizeof(*addr))))
+  while (addr > addrStart && isspace((int)(unsigned char)*(addr-sizeof(*addr))))
     {
       addr--;
     }
 
-  res = xstrndup (addrStart, addr - addrStart);
+  if (addr <= addrStart)
+    res = NULL;
+  else
+    res = xstrndup (addrStart, addr - addrStart);
 
   *addrPtr = (*addrEnd == '\0') ? NULL : addrEnd+1;
   return res;
