@@ -383,6 +383,17 @@ read_line (FILE *input, size_t *max_len_arg)
   return res;
 }
 
+/*
+
+@deftypefn Supplemental char * xstrndup(const char *@var{str}, size_t @var{len})   
+
+Convenience function to copy @var{len} bytes of @var{str} to a newly allocated
+string.  Returns char pointer to new string, or returns NULL if @var{len} < 1
+or @var{str} == NULL.
+
+@end deftypefn
+
+*/
 char *
 xstrndup (const char *str, size_t len)
 {
@@ -392,14 +403,63 @@ xstrndup (const char *str, size_t len)
     }
   else
     {
-      char *res = xmalloc (len + 1);
-      
+      char *res;
+
+      if (len < 1) {
+	len = 0;
+      }
+      res = xmalloc (len + 1);
       memcpy (res, str, len);
       res[len] = '\0';
       return res;
     }
 }
 
+/* Allocate a memory with internal error handling. */
+PTR
+xmalloc(size_t size)
+{
+  void *ptr;
+
+  if (size == (size_t)NULL)
+    size = 1;
+  if ((ptr = malloc(size)) == NULL) {
+    (void)fprintf(stderr, "xmalloc: unable to allocate memory");
+    program_error(__FILE__, __LINE__);
+  }
+  return (ptr);
+}
+
+/* Safely change the size of previously allocated memory area. */
+PTR
+xrealloc(PTR ptr, size_t size)
+{
+  PTR ptr2;
+
+  if (size == (size_t)NULL)
+    size = 1;
+  if ((ptr2 = realloc(ptr, size)) == NULL) {
+    if (ptr != (PTR)NULL)
+      free(ptr);
+    (void)fprintf(stderr, "xrealloc: unable to relocate memory");
+    program_error(__FILE__, __LINE__);
+  }
+  return(ptr2);
+}
+
+/* Save a copy of a string with internal error handling. */
+char *
+xstrdup(const char *str)
+{
+  char *strc;
+
+  if ((strc = strdup(str)) == NULL) {
+    (void)fprintf(stderr, "xstrdup: unable to duplicate a string");
+    program_error(__FILE__, __LINE__);
+  }
+  return(strc);
+}
+
 #ifndef HAVE_MKDIR
 /* mkdir adapted from GNU tar.  */
 
@@ -579,7 +639,7 @@ gnats_strftime (char *s, size_t size, const char *template,
        * and then later be relied on to point to the same data. */
       struct tm btime = *brokentime;
       int result;
-     
+
       /* Count number of %z so we know how much characters to add to the
        * template.  We actually count the number of additional characters.
        * As we are going to replace each "%z" by "+hhmm" (sign, hours,
@@ -680,3 +740,207 @@ value_is_empty (const char *string)
     return TRUE;
   }
 }
+
+/* The following functions are stolen from libiberty library (-liberty) */
+
+#ifndef HAVE_ASPRINTF
+/*
+@deftypefn Extension int asprintf (char **@var{resptr}, const char *@var{format}, ...)
+
+Like @code{sprintf}, but instead of passing a pointer to a buffer, you
+pass a pointer to a pointer.  This function will compute the size of
+the buffer needed, allocate memory with @code{malloc}, and store a
+pointer to the allocated memory in @code{*@var{resptr}}.  The value
+returned is the same as @code{sprintf} would return.  If memory could
+not be allocated, minus one is returned and @code{NULL} is stored in
+@code{*@var{resptr}}.
+
+@end deftypefn
+*/
+
+int
+asprintf VPARAMS ((char **buf, const char *fmt, ...))
+{
+  int status;
+  VA_OPEN (ap, fmt);
+  VA_FIXEDARG (ap, char **, buf);
+  VA_FIXEDARG (ap, const char *, fmt);
+  status = vasprintf (buf, fmt, ap);
+  VA_CLOSE (ap);
+  return status;
+}
+#endif /* HAVE_ASPRINTF */
+
+#ifndef HAVE_VASPRINTF
+/*
+@deftypefn Extension int vasprintf (char **@var{resptr}, const char *@var{format}, va_list @var{args})
+
+Like @code{vsprintf}, but instead of passing a pointer to a buffer,
+you pass a pointer to a pointer.  This function will compute the size
+of the buffer needed, allocate memory with @code{malloc}, and store a
+pointer to the allocated memory in @code{*@var{resptr}}.  The value
+returned is the same as @code{vsprintf} would return.  If memory could
+not be allocated, minus one is returned and @code{NULL} is stored in
+@code{*@var{resptr}}.
+
+@end deftypefn
+*/
+
+/* This is a vasprintf helper */
+static int int_vasprintf PARAMS ((char **, const char *, va_list));
+
+static int
+int_vasprintf (result, format, args)
+     char **result;
+     const char *format;
+     va_list args;
+{
+  const char *p = format;
+  /* Add one to make sure that it is never zero, which might cause malloc
+     to return NULL.  */
+  int total_width = strlen (format) + 1;
+  va_list ap;
+
+#ifdef va_copy
+  va_copy (ap, args);
+#else
+  memcpy ((PTR) &ap, (PTR) &args, sizeof (va_list));
+#endif
+
+  while (*p != '\0')
+    {
+      if (*p++ == '%')
+	{
+	  while (strchr ("-+ #0", *p))
+	    ++p;
+	  if (*p == '*')
+	    {
+	      ++p;
+	      total_width += abs (va_arg (ap, int));
+	    }
+	  else
+	    total_width += strtoul (p, (char **) &p, 10);
+	  if (*p == '.')
+	    {
+	      ++p;
+	      if (*p == '*')
+		{
+		  ++p;
+		  total_width += abs (va_arg (ap, int));
+		}
+	      else
+	      total_width += strtoul (p, (char **) &p, 10);
+	    }
+	  while (strchr ("hlL", *p))
+	    ++p;
+	  /* Should be big enough for any format specifier except %s and floats.  */
+	  total_width += 30;
+	  switch (*p)
+	    {
+	    case 'd':
+	    case 'i':
+	    case 'o':
+	    case 'u':
+	    case 'x':
+	    case 'X':
+	    case 'c':
+	      (void) va_arg (ap, int);
+	      break;
+	    case 'f':
+	    case 'e':
+	    case 'E':
+	    case 'g':
+	    case 'G':
+	      (void) va_arg (ap, double);
+	      /* Since an ieee double can have an exponent of 307, we'll
+		 make the buffer wide enough to cover the gross case. */
+	      total_width += 307;
+	      break;
+	    case 's':
+	      total_width += strlen (va_arg (ap, char *));
+	      break;
+	    case 'p':
+	    case 'n':
+	      (void) va_arg (ap, char *);
+	      break;
+	    }
+	  p++;
+	}
+    }
+#ifdef va_copy
+  va_end (ap);
+#endif
+  *result = (char *) malloc (total_width);
+  if (*result != NULL)
+    return vsprintf (*result, format, args);
+  else
+    return -1;
+}
+
+int
+vasprintf (result, format, args)
+     char **result;
+     const char *format;
+#if defined (_BSD_VA_LIST_) && defined (__FreeBSD__)
+     _BSD_VA_LIST_ args;
+#else
+     va_list args;
+#endif
+{
+  return int_vasprintf (result, format, args);
+}
+#endif /* HAVE_VASPRINTF */
+
+#ifndef HAVE_BASENAME
+/*
+@deftypefn Supplemental char* basename (const char *@var{name})
+
+Returns a pointer to the last component of pathname @var{name}.
+Behavior is undefined if the pathname ends in a directory separator.
+
+@end deftypefn
+*/
+
+#ifndef DIR_SEPARATOR
+#define DIR_SEPARATOR '/'
+#endif
+
+#if defined (_WIN32) || defined (__MSDOS__) || defined (__DJGPP__) || \
+  defined (__OS2__)
+#define HAVE_DOS_BASED_FILE_SYSTEM
+#ifndef DIR_SEPARATOR_2 
+#define DIR_SEPARATOR_2 '\\'
+#endif
+#endif
+
+/* Define IS_DIR_SEPARATOR.  */
+#ifndef DIR_SEPARATOR_2
+# define IS_DIR_SEPARATOR(ch) ((ch) == DIR_SEPARATOR)
+#else /* DIR_SEPARATOR_2 */
+# define IS_DIR_SEPARATOR(ch) \
+	(((ch) == DIR_SEPARATOR) || ((ch) == DIR_SEPARATOR_2))
+#endif /* DIR_SEPARATOR_2 */
+
+/* Note: to be POSIX-2 compliant "name" have a "char *" type. */
+char *
+basename (name)
+     char *name;
+{
+  const char *base;
+
+#if defined (HAVE_DOS_BASED_FILE_SYSTEM)
+  /* Skip over the disk name in MSDOS pathnames. */
+  if (ISALPHA (name[0]) && name[1] == ':') 
+    name += 2;
+#endif
+
+  for (base = name; *name; name++)
+    {
+      if (IS_DIR_SEPARATOR (*name))
+	{
+	  base = name + 1;
+	}
+    }
+  return (char *) base;
+}
+#endif /* HAVE_BASENAME */
