@@ -166,7 +166,7 @@ main (int argc, char **argv)
 
   if (! networkmode)
     {
-      networkmode = gnatsdbHasNetconn ();
+      networkmode = gnatsdbHasNetconn (nameOfDatabase);
     }
 
   if (! networkmode)
@@ -177,7 +177,7 @@ main (int argc, char **argv)
 	  client_print_errors (NULL, err);
 	  exit (1);
 	}
-      pr = readPRWithNum (database, argv[1], 0, &err);
+      pr = readPRWithNum (database, argv[optind], 0, &err);
       if (pr == NULL)
 	{
 	  client_print_errors (database, err);
@@ -192,7 +192,7 @@ main (int argc, char **argv)
 	  client_print_errors (NULL, err);
 	  exit (1);
 	}
-      pr = clientReadPR (argv[1]);
+      pr = clientReadPR (argv[optind]);
     }
   if (pr != NULL)
     {
