@@ -154,7 +154,6 @@ allocPR (const DatabaseInfo database)
 }
 
 /* Get the PR at PATH.  PR is the PR entry to be filled in.
-
    If PRUNE is non-zero, don't read any multitext fields.  */
 static int
 get_pr (PR *pr, const char *path, int prune)
@@ -211,6 +210,33 @@ fillInPR (PR *pr, ErrorDesc *err)
     }
 }
 
+static PR *
+get_pr_from_index (const DatabaseInfo database, const char *prnum,
+		   ErrorDesc *err)
+{
+  PR *pr = getFirstPR (database, err);
+
+  /* If they gave it to us with the category, remove it. */
+  if (( strrchr (prnum, '/')) != NULL)
+    {
+      prnum = strrchr (prnum, '/') + 1;
+    }
+
+  while (pr != NULL && strcmp (prnum, field_value (pr, NUMBER (database))) != 0)
+    {
+      pr = getNextPR (pr);
+    }
+
+  if (pr == NULL)
+    {
+      setError (err, CODE_NONEXISTENT_PR,
+	        "No PR %s listed in the index.", prnum);
+      return NULL;
+    }
+
+  return pr;
+}
+
 /* Initializes PR for reading.  Each line of the PR should be passed
    in via addLineToPR (). */
 
@@ -1348,29 +1374,16 @@ free_pr (PR *pr)
 }
 
 static char *
-get_pr_path (const DatabaseInfo database,const char *prnum, ErrorDesc *err)
+get_pr_path (const DatabaseInfo database, PR *pr, const char *prnum,
+	    ErrorDesc *err)
 {
   char *path = NULL;
   const char *category = NULL;
-  PR *curr_pr_chain = getFirstPR (database, err);
   char *categoryFromIndex = NULL;
 
-  if (curr_pr_chain != NULL)
+  if (pr != NULL)
     {
-      PR *j;
-      for (j = curr_pr_chain ; j != NULL ; j = getNextPR (j))
-	{
-	  if (strcmp (prnum, field_value (j, NUMBER (database))) == 0)
-	    {
-	      category = field_value (j, CATEGORY (database));
-	      break;
-	    }
-	}
-      if (category == NULL)
-	{
-	  setError (err, CODE_NONEXISTENT_PR,
-		    "No PR %s listed in the index.", prnum);
-	}
+      category = field_value (pr, CATEGORY (database));
     }
   else
     {
@@ -1391,9 +1404,10 @@ get_pr_path (const DatabaseInfo database,const char *prnum, ErrorDesc *err)
 }
 
 int
-prExists (const DatabaseInfo database, const char *prID, ErrorDesc *err)
+prExists (const DatabaseInfo database, const char *prnum, ErrorDesc *err)
 {
-  char *path = get_pr_path (database, prID, err);
+  PR *pr = get_pr_from_index (database, prnum, err);
+  char *path = get_pr_path (database, pr, prnum, err);
   int res = 0;
   if (path != NULL)
     {
@@ -1407,50 +1421,87 @@ prExists (const DatabaseInfo database, const char *prID, ErrorDesc *err)
   return res;
 }
 
-char *
-lookup_pr_path (const DatabaseInfo database, const char *prnum, ErrorDesc *err)
+static bool
+pr_file_readable (const char *path, ErrorDesc *err)
 {
-  char *path;
   FILE *fp;
 
-  /* If they gave it to us with the category, remove it. */
-  if (( strrchr (prnum, '/')) != NULL)
+  if (path == NULL)
     {
-      prnum = strrchr (prnum, '/') + 1;
+      return FALSE;
     }
-  path = get_pr_path (database, prnum, err);
-  if ((path == NULL) || ((fp = fopen (path, "r")) == NULL))
+
+  if ((fp = fopen (path, "r")) == NULL)
     {
-      if (path != NULL)
-	{
-	  setError (err, CODE_FILE_ERROR, "Can't open file `%s'", path);
-	}
-      return NULL;
+      setError (err, CODE_FILE_ERROR, "Can't open file `%s'", path);
+      return FALSE;
     }
 
   fclose (fp);
-  return path;
+  return TRUE;
 }
 
 PR *
-readPRWithNum (const DatabaseInfo database, const char *prID,
+readPRWithNum (const DatabaseInfo database, const char *prnum,
 	       int prune, ErrorDesc *err)
 {
   PR *pr = NULL;
-  char *path = lookup_pr_path (database, prID, err);
+  PR *index_pr = get_pr_from_index (database, prnum, err);
+  char *path = get_pr_path (database, index_pr, prnum, err);
 
   if (path != NULL)
     {
-      pr = allocPR (database);
-      if (get_pr (pr, path, prune) == 0)
-	{
-	  free_pr (pr);
-	  pr = NULL;
-	}
+      if (pr_file_readable (path, err))
+        {
+	  pr = allocPR (database);
+	  setPrevPR (pr, getPrevPR (index_pr));
+	  setNextPR (pr, getNextPR (index_pr));
+	  if (get_pr (pr, path, prune) == 0)
+	    {
+	      free_pr (pr);
+	      pr = NULL;
+	    }
+        }
+      free (path);
     }
   return pr;
 }
 
+int
+pr_delete (const DatabaseInfo database, const char *prnum, ErrorDesc *err)
+{
+  PR *pr;
+  char *path = NULL;
+
+  pr = get_pr_from_index (database, prnum, err);
+  path = get_pr_path (database, pr, prnum, err);
+
+  if (path == NULL || !pr_file_readable (path, err))
+    {
+      if (path != NULL)
+	{
+	  free (path);
+	}
+      return -1;
+    }
+
+  if (removePRFromIndex (database, prnum, err))
+    {
+      free (path);
+      return -5;
+    }
+  
+  if (unlink (path))
+    {
+      setError (err, CODE_FILE_ERROR, "Unable to unlink file %s\n", path);
+      free (path);
+      return -6;
+    }
+
+  free (path);
+  return 1;
+}
+
 void
 freeInputTemplate (InputTemplate *template)
 {
