@@ -1,5 +1,5 @@
 /* Perform queries on the contents of a GNATS database.
-   Copyright (C) 2001 Milan Zamazal
+   Copyright (C) 2001, 2002 Milan Zamazal
    Copyright (C) 1993, 94, 95, 96, 1997, 2000 Free Software Foundation, Inc.
    Contributed by Brendan Kehoe (brendan@cygnus.com).
    Further hacked by Milan Zamazal (pdm@zamazal.org).
@@ -130,7 +130,10 @@ Query database.\n\
           [--responsible-address address]\n\
           [--list-databases] [--list-fields] [--list-input-fields]\n\
           [--field-type type] [--field-description description]\n\
-          [--valid-values values]\n\
+          [--field-flags field]\n\
+          [--adm-field field] [--adm-subfield subfield]\n\
+          [--adm-key key]\n",
+"          [--valid-values values]\n\
           [--format format | -f format] [--full | -F] [--summary | -q]\n\
           [--and | -&] [--or | -|] [--expr expr]\n\
           [--help | -h] [--version | -V] [--debug | -D]\n\
@@ -141,6 +144,7 @@ Query database.\n\
   "Network-mode-only options:\n\
           [--host host | -H host] [--port port]\n\
           [--user user | -v user] [--passwd passwd | -w passwd]\n\
+          [--print-server-addr]\n\
 \n",
   "Deprecated options:\n\
           [--list-categories | -j] [--list-states | -T]\n\
@@ -611,7 +615,7 @@ main (int argc, char **argv)
 
   if (! is_network_client)
     {
-      is_network_client = gnatsdbHasNetconn ();
+      is_network_client = gnatsdbHasNetconn (nameOfDatabase);
     }
 
   if (printServerAddr)
